/**
* @license Apache-2.0
*
* Copyright (c) 2019 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

/**
* Open a file.
*
* @module @stdlib/fs/open
*
* @example
* var closeSync = require( '@stdlib/fs/close' ).sync;
* var open = require( '@stdlib/fs/open' );
*
* function onOpen( error, fd ) {
*     if ( error ) {
*         throw error;
*     }
*     closeSync( fd );
* }
* open( __filename, onOpen );
*
* @example
* var closeSync = require( '@stdlib/fs/close' ).sync;
* var openSync = require( '@stdlib/fs/open' ).sync;
*
* var fd = openSync( __filename );
* if ( fd instanceof Error ) {
*     throw fd;
* }
* closeSync( fd );
*/

// MODULES //

var setReadOnly = require( '@stdlib/utils/define-nonenumerable-read-only-property' );
var main = require( './main.js' );
var sync = require( './sync.js' );


// MAIN //

setReadOnly( main, 'sync', sync );


// EXPORTS //

module.exports = main;
